@@ -8,6 +8,6 @@ sub lock { $_[0] -> {_locked}++; return $_[0]->{_locked}; }
 
 sub unlock { return $_[0] -> {_locked}-- if $_[0]->{_locked}; }
 
-sub is_locked { return $_[0] -> {_locked} > 0; }
+sub is_locked { return defined($_[0] -> {_locked}) && $_[0] -> {_locked} > 0; }
 
 1;
